/**
 * @Author: René Vos 
 * @email: rene.rpv@gmail.com
 * @Date: 17-5-2011
 * 
 * Version 2.0
 *
 * @Author: Hielke de Haan 
 * @email: Hielke@hdehaan.com
 * @Date: 18-5-2011
 * 
 */
using System;

using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;

using GHIElectronics.NETMF.FEZ;

namespace MD2x
{
    public class MotorControl
    {
        // Constantes
        private const int TRANSACTIONEXECUTETIMEOUT = 1000;

        // Attributs
        private I2CDevice.Configuration configMD2x;
        private I2CDevice.I2CTransaction[] transactions;

        // Registres de la carte MD2x
        private byte speed1 = 0, speed2Turn = 0;
        private byte softrev = 0;
        private byte voltage = 0;
        private byte current1 = 0, current2=0;
        private byte acceleration = 0;
        private byte mode=0;
        private byte command = 0;
        private Int32 Encod1=0, Encod2=0;

        // Constructeur
        public MotorControl(byte I2C_Address, UInt16 Freq)
        {
            configMD2x = new I2CDevice.Configuration(I2C_Address, Freq);
        }
        
        // Propriétés
        public I2CDevice.Configuration Config
        {
            get { return configMD2x; }
        }
        public Byte Speed1
        {
            get { return speed1; }
        }
        public Byte Speed2Turn
        {
            get { return speed2Turn; }
        }
        public Int32 Encodeur1
        {
            get { return Encod1; }
        }
        public Int32 Encodeur2
        {
            get { return Encod2; }
        }
        public Byte Voltage
        {
            get { return voltage; }
        }
        public Byte Current1
        {
            get { return current1; }
        }
        public Byte SoftRev
        {
            get { return softrev; }
        }
        public Byte Current2
        {
            get { return current2; }
        }
        public Byte AccelerationRate
        {
            get { return acceleration; }
        }
        public Byte Mode
        {
            get { return mode; }
        }
        public Byte Command
        {
            get { return command; }
        }

       

       /* public Boolean Initialize()
        {
            i2cConfig = new I2CDevice.Configuration(DEVICEADDRESS, CLOCKRATE);
            if (i2cConfig != null)
            {
                i2cBus = new I2CDevice(i2cConfig);
                if (i2cBus != null)
                {
                    return true;
                }
            }
            return false;
        }
        */


        // Méthodes publiques
        public Boolean SetSpeed(int speed)
        {
            return false;
        }

        public Boolean SetDirection(Boolean forward)
        {
            return false;   
        }

        /// <summary>
        /// Lecture des 17 registres de la carte MD2x
        /// </summary>
        /// <param name="i2cBus"></param>
        /// <returns></returns>
        public int ReadRegister(I2CDevice I2CBus)
        {
            // Buffer d'écriture
            byte[] outBuffer = new byte[] { 0 };
            I2CDevice.I2CWriteTransaction writeTransaction = I2CDevice.CreateWriteTransaction(outBuffer);

            // Buffer de lecture
            byte[] inBuffer = new byte[17];
            I2CDevice.I2CReadTransaction readTransaction = I2CDevice.CreateReadTransaction(inBuffer);

            // Exécution des transactions
            transactions = new I2CDevice.I2CTransaction[] { writeTransaction, readTransaction };
            I2CBus.Config = configMD2x; // Connexion virtuelle de la carte MD2x au bus I2C

            if (I2CBus.Execute(transactions, TRANSACTIONEXECUTETIMEOUT) != 0)
            {
                // Success
                Debug.Print("Received the first data from at device " + I2CBus.Config.Address + ": " + ((int)inBuffer[0]).ToString());
                softrev = inBuffer[13];
                Encod1 = (Int32)(inBuffer[2] << 24) | (Int32)(inBuffer[3] << 16) | (Int32)(inBuffer[4] << 8) | (Int32)(inBuffer[5]);
                //Debug.Print("Encodeur1=" + Encodeur1.ToString());
                Encod2 = (Int32)(inBuffer[6] << 24) | (Int32)(inBuffer[7] << 16) | (Int32)(inBuffer[8] << 8) | (Int32)(inBuffer[9]);
                //Debug.Print("Encodeur1=" + Encodeur1.ToString());

            }
            else
            {
                // Failed
                Debug.Print("Failed to execute transaction at device: " + I2CBus.Config.Address + ".");
            }

            return 1;
        }

        public int WriteRegister(I2CDevice i2cBus, byte address, byte value)
        {
            transactions = new I2CDevice.I2CTransaction[1];

            byte[] WriteData = new byte[2] { address, value};
            transactions[0] = I2CDevice.CreateWriteTransaction(WriteData);

            if (i2cBus.Execute(transactions, TRANSACTIONEXECUTETIMEOUT) != 0)
            {
                // Success
                Debug.Print("Succesfully wrote data at device " + i2cBus.Config.Address + ": Value: " + ((int)value).ToString() + " to register: " + ((int)address).ToString());
            }
            else
            {
                // Failed
                Debug.Print("Failed to execute transaction at device: " + i2cBus.Config.Address + ".");
            }
            return 1;
        }
        public void Forward(I2CDevice i2cBus, int i)
        {
            if (i == 0) Stop(i2cBus);
            else
            {
                i = i + 27;
                if (i >= 128) i = 128;

                int leftspeed = 128 + i;
                int rightspeed = 128 - i;

                WriteRegister(i2cBus, 0x01, (byte)rightspeed);
                WriteRegister(i2cBus, 0x00, (byte)leftspeed);
            }
        }
        public void Backward(I2CDevice i2cBus, int i)
        {
            if (i == 0) Stop(i2cBus);
            else
            {
                i = i + 27;
                if (i >= 128) i = 128;

                int leftspeed = 128 - i;
                int rightspeed = 128 + i;

                WriteRegister(i2cBus, 0x01, (byte)rightspeed);
                WriteRegister(i2cBus, 0x02, (byte)leftspeed);
            }
        }
        public void TurnRight(I2CDevice i2cBus)
        {
            int leftspeed = 254;
            int rightspeed = 254;

            WriteRegister(i2cBus, 0x01, (byte)rightspeed);
            WriteRegister(i2cBus, 0x02, (byte)leftspeed);
        }
        public void TurnLeft(I2CDevice i2cBus)
        {
            int leftspeed = 0;
            int rightspeed = 0;

            WriteRegister(i2cBus, 0x01, (byte)rightspeed);
            WriteRegister(i2cBus, 0x02, (byte)leftspeed);
        }
        public void Stop(I2CDevice i2cBus)
        {
            WriteRegister(i2cBus, 0x00, 0x80);
            WriteRegister(i2cBus, 0x01, 0x80);
        }
        public void ReadParramters(I2CDevice i2cBus, int i)
        {
            /*ReadRegister(i2cBus,0x00);
            ReadRegister(0x01);
            ReadRegister(0x02);
            ReadRegister(0x03);
            ReadRegister(0x04);
            ReadRegister(0x05);
            ReadRegister(0x07);*/

        }

    }
 }
